%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Combine bundle-adjusted intrinsic and extrinsic calibration with
% refractive calibration
bundleFile = 'calib/extrinsic/bundle_rescaled.out';
bundleImageList = 'calib/extrinsic/list.txt';
outFile = 'bundle_combined.out';

% Average refractive indices for normal light
mu = [1.0; 1.491; 1.333];

% This file is assumed to contain only air-calibration data. Refractive
% parameters N, Mu, D will be read from the refractive calibration files.
[numCams, F, K, R, T] = readBundleCams(bundleFile);
[X, Color, KeyIdx, KeyCoord] = readBundlePoints(bundleFile);

N = zeros(numCams, 3);
Mu = cell(numCams, 1);
D = cell(numCams, 1);

fid = fopen(bundleImageList, 'r');
camIdx = 1;

while 1
    tline = fgetl(fid);
    if ~ischar(tline), break, end

    [calibFile, imgName, camID, imgFile] = getRefractiveCalib(tline);
    
    load(calibFile, 'XFinal', 'XInit', 'glassThickness', 'muGlassRed', 'muWaterRed');
    
    D{camIdx} = [mean(XFinal(:,9)); glassThickness];
    %D{camIdx} = [mean(XInit(:,9)); glassThickness];
    Mu{camIdx} = mu;
    % Change from LHS coord system to RHS coord system (??), and flip y
    % axis
    N(camIdx,:) = mean(XFinal(:,10:12), 1)' .* [1; -1; -1];
    %N(camIdx,:) = mean(XInit(:,10:12), 1)' .* [1; -1; -1];
    
    % Bundler expects the refraction-corrected .out files to have an
    % 'adjusted' focal length
    F(camIdx) = F(camIdx) * mu(end)/mu(1);
    
    %mean(XFinal(:,10:12), 1)'
    
    camIdx = camIdx + 1;
end

if camIdx ~= (numCams + 1)
    error(['Mismatched number of cameras from ' bundleImageList ' and ' bundleFile]);
end

printBundleCamsPoints(outFile, numCams, F, K, R, T, N, Mu, D, X, Color, KeyIdx, KeyCoord);
