################################################################################
# Thesis title: Underwater Camera Calibration and 3D Reconstruction
# http://hdl.handle.net/10402/era.38695
# Author: Timothy Yau
################################################################################

matlab -wait -nodesktop -nosplash -r "removePointsData('../bundle_combined.out', 'bundle_combined.out'); exit"

cat << 'EOF' > options.txt
--match_table matches.init.txt
--output bundle.out
--output_all bundle_
--output_dir bundle_refrac
--fixed_some_refract_layers 0x2
--fmatrix_threshold 512.0
--homography_threshold 512.0
--fixed_cam_t
--fixed_cam_r
--bundle bundle_combined.out
--rerun_bundle
EOF

mkdir bundle_refrac

for file in ./*.key.gz.orig ; do         
  if [ -e "$file" ] ; then   
     cp "$file" "${file:0:(${#file}-5)}"
  fi
done

matlab -wait -nodesktop -nosplash -r "undistortKeys; exit"

perl ../../../scripts/makeImageList.pl
rm list.txt
mv newlist.txt list.txt
rm matches.init.txt
mv newmatches.init.txt matches.init.txt

matlab -wait -nodesktop -nosplash -r "updateListParams('list.txt', 'bundle_combined.out'); exit"

rm constraints.txt
../../../bundler-refrac/bin/bundler.exe list.txt --options_file options.txt;

matlab -wait -nodesktop -nosplash -r "undistortImages('bundle_refrac/bundle.out'); exit"

# Temporarily rename original image files
for file in ./*.jpg ; do         
  if [ -e "$file" ] ; then   
     mv "$file" "$file.tmp"
  fi
done

# Put undistorted files in place of originals
for file in ./*.jpg.undist ; do         
  if [ -e "$file" ] ; then   
     mv "$file" "${file:0:(${#file}-7)}"
  fi
done

../../../bundler-refrac/bin/Bundle2PMVS.exe list.txt bundle_tmp.out pmvs;

sed 's/minImageNum 3/minImageNum 2/' < pmvs/pmvs_options.txt > pmvs_options.txt.tmp
echo 'maxAngle 0.1' >> pmvs_options.txt.tmp
mv pmvs_options.txt.tmp pmvs/pmvs_options.txt

sed 's/ # Edit this line before running/..\/..\/..\/bundler-refrac\/bin/' < pmvs/prep_pmvs.sh > prep_pmvs.sh.tmp
mv prep_pmvs.sh.tmp pmvs/prep_pmvs.sh

pmvs/prep_pmvs.sh


# Put the 'undist' suffix back again
for file in ./*.jpg ; do         
  if [ -e "$file" ] ; then   
     mv "$file" "$file.undist"
  fi
done

# Replace the original images
for file in ./*.jpg.tmp ; do         
  if [ -e "$file" ] ; then   
     mv "$file" "${file:0:(${#file}-4)}"
  fi
done

# Put segmentation masks in place
mkdir pmvs/edges
cp edge_undist/* pmvs/edges

