%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = updateListParams(listFile, bundleFile)
    % Read bundle file params and insert them into list.txt, assuming that the
    % cameras are listed in the same order.
    
    [numCams, F, K, R, T, N, Mu, D] = readBundleCams(bundleFile);
    
    listEntries = cell(numCams,1);
    
    fid = fopen(listFile, 'r');
    i = 1;
    tline = fgetl(fid);
    while ischar(tline)
        if ~isempty(tline)
            listEntries{i} = tline;
            i = i + 1;
        end
        tline = fgetl(fid);
    end
    fclose(fid);
    
    if i > (numCams+1)
        error('Number of cameras doesn''t match');
    end
    
    fid = fopen(listFile, 'w');
    for i=1:numCams
        imgFile = strtok(listEntries{i}, ' ');
        numRefrac = numel(D{i});
        
        if numRefrac > 0
            F(i) = F(i) * Mu{i}(1) / Mu{i}(end);
        end
        
        fprintf(fid, '%s 0 %f flat_refract %f %f %f %i', imgFile, F(i), N(i,1), N(i,2), N(i,3), numRefrac);
        
        for j=1:numRefrac
            fprintf(fid, ' %f %f', Mu{i}(j), D{i}(j));
        end
        
        if numRefrac > 0
            fprintf(fid, ' %f', Mu{i}(numRefrac+1));
        end
        
        fprintf(fid, '\n');
    end
    fclose(fid);
    