%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%
% This file is adapted and modified from code accompanying the paper:
%   A. Agrawal, S. Ramalingam, Y. Taguchi, and V. Chari. A theory of 
%   multilayer flat refractive geometry. CVPR 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute 3D error between the back-projected ray and the given 3D point
% for two layers Case 2

function [C] = Compute3DErrorTwoLayerCase3(t,xy,XYZ,R,n,mu1,mu2,tau0,tau1,xyb,mu1b,mu2b)

if nargin < 10
    xyb = [];
    mu1b = [];
    mu2b = [];
end

N = size(XYZ,2);

% transform points
P = R*XYZ + t*ones(1,N);

if isempty(xyb)
    Mu1 = repmat(mu1,N,1);
    Mu2 = repmat(mu2,N,1);
else
    Mu1 = [repmat(mu1,N,1); repmat(mu1b,N,1)];
    Mu2 = [repmat(mu2,N,1); repmat(mu2b,N,1)];
    xy = [xy, xyb];
    P = [P, P];
    N = N * 2;
end

e = zeros(N,1);

d0 = tau0;
d1 = tau1;

for ii = 1:N
    
    pt = P(:,ii);
    v = xy(:,ii);
    
    q1 = -d0*v/(v'*n);
    
    
    v1 = RefractedRay(v,n,1,Mu1(ii));
        
    %find q2
    
    q2 = q1 - d1*v1/(v1'*n);
    
    %final refracted ray
    v2 = RefractedRay(v1,n,Mu1(ii),Mu2(ii));
    
    e(ii,1) = norm(cross(v2,q2 - pt))/norm(v2);
    
end


C = median(e.^2);

