%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%
% This file is adapted and modified from code accompanying the paper:
%   A. Agrawal, S. Ramalingam, Y. Taguchi, and V. Chari. A theory of 
%   multilayer flat refractive geometry. CVPR 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Find vector of reprojection errors for non-linear refinement 
% for two layers Case 3
function [C] = CostFuncTwoLayersCase3Disp(x,rayDnoise,rayDnoise2,XYZ,KK,mu1r,mu2r,mu1b,mu2b,tau2,KKb,tau1)

x = x(:);

n = x(1:2);
w = x(3:5);
t = x(6:8);

if nargin < 10
    tau2 = x(10);
end

if nargin < 11
    KKb = KK;
end

if nargin < 12
    tau1 = x(9);
end

n = [n ; -1];
n = n/norm(n);
R = RPY2Rot(w);

N = size(XYZ,2);
C = zeros(4*N,1);

for ii = 1:N 
    p = R*XYZ(:,ii) + t;
    
    M = SolveFlatRefractionPointTwoLayerCase3(tau1,tau2,n,mu1r,mu2r,p);
    M = KK*M;
    px1 = M(1)/M(3);
    py1 = M(2)/M(3);
    px = rayDnoise(1,ii);
    py = rayDnoise(2,ii);
    C(4*ii-1,1) = px-px1;
    C(4*ii,1) = py-py1;
    
    M = SolveFlatRefractionPointTwoLayerCase3(tau1,tau2,n,mu1b,mu2b,p);
    M = KKb*M;
    px1 = M(1)/M(3);
    py1 = M(2)/M(3);
    px = rayDnoise2(1,ii);
    py = rayDnoise2(2,ii);
    C(4*ii-3,1) = px-px1;
    C(4*ii-2,1) = py-py1;
end

