%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%
% This file is adapted and modified from code accompanying the paper:
%   A. Agrawal, S. Ramalingam, Y. Taguchi, and V. Chari. A theory of 
%   multilayer flat refractive geometry. CVPR 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [xreproj, xreprojb] = PlotReprojectedPointsNoimgCase3Disp(x,CameraRays,CameraRaysb,Pall,mu1r,mu2r,mu1b,mu2b,tau2,KK,KKb)

if nargin < 11
    KKb = KK;
end

N = size(Pall,2);

n = [x(1:2) ; -1];
n = n/norm(n);

figure;hold on;
plot(CameraRays(1,:)',CameraRays(2,:)','*', 'Color', [1.0,0.4,0.4]);hold on;
plot(CameraRaysb(1,:)',CameraRaysb(2,:)','*', 'Color', [0.4,0.4,1.0]);hold on;
R1 = RPY2Rot(x(3:5));
t1 = x(6:8);
tau1 = x(9);
if isempty(tau2)
    tau2 = x(10);
end
P = R1*Pall + t1*ones(1,N);

clear xreproj

xreproj = zeros(2,N);
xreprojb = zeros(2,N);

% find the image reprojection point for each 3D point
for ii = 1:N
    p = P(:,ii);
    M = SolveFlatRefractionPointTwoLayerCase3(tau1,tau2,n,mu1r,mu2r,p);
    M = KK*M;
    xreproj(1,ii) = M(1)/M(3);
    xreproj(2,ii) = M(2)/M(3);
    
    M = SolveFlatRefractionPointTwoLayerCase3(tau1,tau2,n,mu1b,mu2b,p);
    M = KKb*M;
    xreprojb(1,ii) = M(1)/M(3);
    xreprojb(2,ii) = M(2)/M(3);
end
plot(xreproj(1,:)',xreproj(2,:)','rs');
plot(xreprojb(1,:)',xreprojb(2,:)','bs');

legend('detected points (red)', 'detected points (blue)', 'reprojected 3D points (red)', 'reprojected 3D points (blue)');


