%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%
% This file is adapted and modified from code accompanying the paper:
%   A. Agrawal, S. Ramalingam, Y. Taguchi, and V. Chari. A theory of 
%   multilayer flat refractive geometry. CVPR 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% compute the total reprojection error for all 3D points using estimated
% calibration parameteres

function [ee, er, eb] = ReprojectionErrorTwoLayersCase3Disp(n,w,t,tau1,tau2,rayDnoise,XYZ,KK,mu1,mu2,rayDnoiseb,mu1b,mu2b,KKb)

n = n/norm(n);
R = RPY2Rot(w);


N = size(XYZ,2);
Cr = zeros(2*N,1);
Cb = zeros(2*N,1);

if nargin < 14
    KKb = KK;
end

for ii = 1:N
    
    p = R*XYZ(:,ii) + t;
    
    M = SolveFlatRefractionPointTwoLayerCase3(tau1,tau2,n,mu1,mu2,p);
    M = KK*M;

    px1 = M(1)/M(3);
    py1 = M(2)/M(3);
    
    px = rayDnoise(1,ii);
    py = rayDnoise(2,ii);
    
    Cr(2*ii-1,1) = px-px1;
    Cr(2*ii,1) = py-py1;
    
    M = SolveFlatRefractionPointTwoLayerCase3(tau1,tau2,n,mu1b,mu2b,p);
    M = KKb*M;

    px1 = M(1)/M(3);
    py1 = M(2)/M(3);
    
    px = rayDnoiseb(1,ii);
    py = rayDnoiseb(2,ii);
    
    Cb(2*ii-1,1) = px-px1;
    Cb(2*ii,1) = py-py1;
end

C = [Cr(:); Cb(:)];


ee = sqrt(mean(C.^2));
er = sqrt(mean(Cr.^2));
eb = sqrt(mean(Cb.^2));






