%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Xrm, Xbm, blueScale, bluekc, blueShift] = adjustRadialDistortion2(Xr, Xb, calibRed, calibBlue)
%     figPos = [10, 10, 500, 400];
    
    load(calibRed, 'fc', 'cc', 'kc', 'alpha_c', 'nx', 'ny');
    imgWidth = nx;
    imgHeight = ny;
    [Xrm, KKr] = undistortAndCenterPoints(Xr,fc,cc,kc,alpha_c,imgWidth,imgHeight);
    clear fc cc kc alpha_c;
    
%     figure('Position',figPos), axis equal, hold on;
%     plot(Xrm(:,1), Xrm(:,2), 'r*');
%     title('Undistorted Red Points');
% %    xlim([0, 1024]); ylim([0, 768]);
%     hold off;
    
    load(calibBlue, 'fc', 'cc', 'kc', 'alpha_c');
    [Xbm, KKb] = undistortAndCenterPoints(Xb,fc,cc,kc,alpha_c,imgWidth,imgHeight);
    clear fc cc kc alpha_c;
    fcb = [KKb(1,1); KKb(2,2)];
    ccb = [KKb(1,3); KKb(2,3)];
    
%     figure('Position',figPos), axis equal, hold on;
%     plot(Xb(:,1), Xb(:,2), 'bo');
%     title('Undistorted Blue Points');
% %    xlim([0, 1024]); ylim([0, 768]);
%     hold off;
    
%     figure('Position',figPos), axis equal, hold on;
%     plot(Xrm(:,1), Xrm(:,2), 'r*');
%     plot(Xbm(:,1), Xbm(:,2), 'bo');
%     title('Undistorted Points');
%     hold off;
    
    % Adjust x and y scale, center shift, and radial distortion
    x0 = [[1;1]; [0;0]; zeros(4,1)];
    
    optimFunc = @(x) undistortOptimFunc(Xrm, Xbm, x(1:2), x(3:4), [x(5:8); 0], fcb, ccb);    
    
    options = optimset('Display', 'iter', 'MaxFunEvals', 500, 'MaxIter', 50);
    %options = optimset('MaxFunEvals', 40000, 'MaxIter', 40000);
    [x, fval] = lsqnonlin(optimFunc, x0, [], [], options);

    blueScale = x(1:2);
    blueShift = x(3:4);
    bluekc = [x(5:8); 0];
    Xbm = undistortAndScale(Xbm, blueScale, blueShift, fcb, ccb, bluekc, 0);

%     figure('Position',figPos), axis equal, hold on;
%     plot(Xrm(:,1), Xrm(:,2), 'r*');
%     plot(Xbm(:,1), Xbm(:,2), 'bo');
%     title('Final Alignment');
%     hold off;
%     
%     quiverRange = 1:1000;
%     figure('Position',figPos), axis equal, hold on;
%     quiver(Xrm(quiverRange,1), Xrm(quiverRange,2), Xbm(quiverRange,1) - Xrm(quiverRange,1), Xbm(quiverRange,2) - Xrm(quiverRange,2), 0);
%     title('Final Alignment Error');
%     %xlim([800,1350]); ylim([550, 875]);
%     hold off;
    
    meanError = mean(sqrt(sum((Xrm - Xbm).^2,2)))
    
function [v] = undistortOptimFunc(Xr, Xb, scale, shift, warp, fc, cc)
    Xb = undistortAndScale(Xb, scale, shift, fc, cc, warp, 0);
    v = sqrt(sum((Xr - Xb).^2,2));
    