%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Refraction axis estimation using dispersion
function [A, axisPt] = axisFromDisp(Ximg, Ximgb, K, plotFile, figProps)
    SetDefaultValue(4, 'plotFile', []);
    SetDefaultValue(5, 'figProps', cell(0));  

    sampleRadius = 100; % in pixels
    numPts = size(Ximg, 1);
    
    Fx = zeros(numPts,1);
    Fy = zeros(numPts,1);
    Fu = zeros(numPts,1);
    Fv = zeros(numPts,1);

    for pointNum = 1:numPts
        samplePt = Ximg(pointNum,:);

        if sampleRadius > 0
            ptsDist = Ximg - repmat(samplePt,numPts,1);
            ptsDist = sqrt(sum(ptsDist.^2,2));

            ptsForAvg = (ptsDist <= sampleRadius);

            [p, v] = convergentAvg(Ximgb(ptsForAvg,1:2), Ximg(ptsForAvg,1:2) - Ximgb(ptsForAvg,1:2));
        else
            p = Ximgb(pointNum,1:2);
            v = Ximg(pointNum,1:2) - Ximgb(pointNum,1:2);
        end
        
        Fx(pointNum) = p(1);
        Fy(pointNum) = p(2);
        Fu(pointNum) = v(1);
        Fv(pointNum) = v(2);
    end
    
    estAxisPt = estConvergentPoint3D([Fx,Fy], [Fu,Fv], K);
    estAxisPt = estAxisPt / estAxisPt(3);

    A = K \ estAxisPt;
    A = A / norm(A);

    axisPt = estAxisPt(1:2);
    
    if ~isempty(plotFile)
        figure; axis equal; hold on;
        set(gca, 'YDir', 'reverse');
%         xlimits = [-0.5, 1599.5];
%         ylimits = [-0.5, 900.5];
%         xlim(xlimits);
%         ylim(ylimits);
        lineScale = 300;
        for i=1:length(Fx)
            plot([Fx(i), Fx(i)+lineScale*Fu(i)], [Fy(i), Fy(i)+lineScale*Fv(i)], 'Color', [0.7, 0.7, 0.7]);
        end
        quiver(Fx, Fy, Fu, Fv);
        plot(estAxisPt(1), estAxisPt(2), 'ro');
        hold off;
        
        if ~isempty(figProps)
            set(gcf, figProps{:});
        end
        
        saveas(gcf, plotFile);
        close;
    end
    