%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [y3, y2] = backProj2(y1, sx, e, t, r1, r2)
% Back projection with refraction through two interfaces, such as
% air->glass->water. The first interface is on the y axis, and the second
% one is to the left by a distance t. 
%
% The eye point is a distance e to the right of the y axis. y1 is the 
% y-coordinate of a point on the y axis, and sx is the distance to the 
% object from the y axis (must be greater than t). y2 and y3 are the 
% projected points on the second interface and on the object respectively.
% sx and e must be positive.
%
% r1 is the refractive index ratio of the second material to the first, and
% r2 is the ratio of the third material to the first. 
% E.g. for air->glass->water this would be something like r1=1.517,
% r2=1.333

u2 = (t * y1 / r1) / sqrt(e^2 + (1 - 1/(r1^2)) * y1^2);
u3 = ((sx - t) * u2 / (r2 / r1)) / sqrt(t^2 + (1 - 1 / ((r2 / r1)^2)) * u2^2);

y2 = y1 + u2;
y3 = y1 + u2 + u3;
