%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = cleanPointsData2()
    global allImages camIDs seqNums fileExt sep PointDataR PointDataB;
    global camIDIdx seqNumIdx;
    global pointsDataFile;
    
    pointsDataFile = 'GridPointData.mat';
    
    copyfile(pointsDataFile, [pointsDataFile '.bak']);
    load(pointsDataFile, 'PointDataR', 'PointDataB', 'allImages', 'camIDs', 'seqNums', 'fileExt', 'sep');
    
    camIDIdx = 0;
    seqNumIdx = 1;
    
    figure;
    nextImage();   
end

function [] = nextImage(hObj,event)
    global seqNums camIDs camIDIdx seqNumIdx pointsDataFile PointDataR PointDataB;
    
    nextImgDisplayed = false;
    
    while seqNumIdx <= numel(seqNums)
        seqNumIdx
        while camIDIdx < numel(camIDs)
            camIDIdx = camIDIdx + 1
            
            nextImgDisplayed = displayPoints();
            
            if nextImgDisplayed
                break;
            end
        end
        
        if nextImgDisplayed
            break;
        else
            seqNumIdx = seqNumIdx + 1;
            camIDIdx = 0;
        end
    end
    
    if seqNumIdx > numel(seqNums)
        close;
        save(pointsDataFile, 'PointDataR', 'PointDataB', '-append');
        disp('All done.');
    end
end

function [fileFound] = displayPoints()
    global seqNums camIDs camIDIdx seqNumIdx sep fileExt curImage PointDataR PointDataB selPtsR selPtsB figHandle;
    
    %figPos = get(0,'ScreenSize');
    figTitle = 'Click points to delete. Right-click for bounding box select.';
    
    imgFile = [seqNums{seqNumIdx} sep camIDs{camIDIdx} '.' fileExt];
    
    if exist(imgFile, 'file')
        fileFound = true;

        curImage = imread(imgFile);
        
        selPtsR = true(1,size(PointDataR{camIDIdx,seqNumIdx},1));
        selPtsB = true(1,size(PointDataB{camIDIdx,seqNumIdx},1));
        
        close;
        figHandle = figure('name', imgFile, 'units','normalized','outerposition',[0 0 1 1]);
        title(figTitle);
        
        uicontrol('Style', 'pushbutton', 'String', 'Accept',...
                  'Position', [20 40 50 20], 'Callback', @acceptImage);
              
        uicontrol('Style', 'pushbutton', 'String', 'Reject',...
                  'Position', [80 40 50 20], 'Callback', @rejectImage);
              
        uicontrol('Style', 'pushbutton', 'String', 'Reset Red',...
                  'Position', [140 40 70 20],...
                  'Callback', @resetRed);
              
        uicontrol('Style', 'pushbutton', 'String', 'Reset Blue',...
                  'Position', [220 40 70 20],...
                  'Callback', @resetBlue);
        
        set(gca, 'buttondownfcn', @buttonDown);
              
        updateFigure();
    else
        fileFound = false;
    end
end

function resetRed(obj,evnt)
    global selPtsR;
    selPtsR(:) = true;
    updateFigure();
end

function resetBlue(obj,evnt)
    global selPtsB;
    selPtsB(:) = true;
    updateFigure();
end

function updateFigure
    global camIDIdx seqNumIdx PointDataR PointDataB curImage selPtsR selPtsB;
    
    cla;
%     xlim('auto');
%     ylim('auto');
    axis equal; hold on;
    imagesc(curImage, 'hittest', 'on', 'buttondownfcn', @buttonDown), colormap gray;
    
    PtsR = PointDataR{camIDIdx,seqNumIdx};
    PtsB = PointDataB{camIDIdx,seqNumIdx};
    
    plot(PtsR(selPtsR,1), PtsR(selPtsR,2), 'r*', 'hittest', 'on', 'buttondownfcn', @buttonDown);
    plot(PtsB(selPtsB,1), PtsB(selPtsB,2), 'bo', 'hittest', 'on', 'buttondownfcn', @buttonDown);

    hold off;
end

function buttonDown(src,evnt)
    disp('Button down!');

    global selPtsR selPtsB PointDataR PointDataB camIDIdx seqNumIdx figHandle;

    threshold = 5/1024; % in normalized units
    
    button = get(figHandle,'SelectionType');
    point = get(gca,'CurrentPoint');
    
    x = point(1,1);
    y = point(1,2);
    
    PtsR = PointDataR{camIDIdx,seqNumIdx};
    PtsB = PointDataB{camIDIdx,seqNumIdx};
    numPtsR = size(PtsR,1);
    numPtsB = size(PtsB,1);
    
    figXLimits = xlim;
    figYLimits = ylim;

    figWidth = figXLimits(2) - figXLimits(1);
    figHeight = figYLimits(2) - figYLimits(1);
    figMaxDim = max([figWidth, figHeight]);
    
    if strcmp(button, 'normal')
        distR = sqrt(sum((PtsR - ones(numPtsR,1)*[x,y]).^2,2)) / figMaxDim;
        distB = sqrt(sum((PtsB - ones(numPtsB,1)*[x,y]).^2,2)) / figMaxDim;

        distR(~selPtsR) = threshold + 1;
        distB(~selPtsB) = threshold + 1;

        [minDistR, minIdxR] = min(distR);
        [minDistB, minIdxB] = min(distB);

        if minDistR < minDistB && minDistR <= threshold
            selPtsR(minIdxR) = false;
        elseif minDistB <= threshold
            selPtsB(minIdxB) = false;
        end
    elseif strcmp(button, 'alt')
        hold on;
        plot([figXLimits(1), figXLimits(2)], [y, y], 'y');
        plot([x, x], [figYLimits(1), figYLimits(2)], 'y');
        hold off;

        [x2,y2] = ginput(1);

        if x2 < x
            tmp = x; x = x2; x2 = tmp;
        end
        if y2 < y
            tmp = y; y = y2; y2 = tmp;
        end

        inBoundsR = (PtsR(:,1) >= x) & (PtsR(:,1) <= x2) & (PtsR(:,2) >= y) & (PtsR(:,2) <= y2);
        inBoundsB = (PtsB(:,1) >= x) & (PtsB(:,1) <= x2) & (PtsB(:,2) >= y) & (PtsB(:,2) <= y2);

        selPtsR(~inBoundsR) = false;
        selPtsB(~inBoundsB) = false;
    end
    
    updateFigure();
end

function acceptImage(obj,evnt)
    global selPtsR selPtsB PointDataR PointDataB camIDIdx seqNumIdx;
    
    PointDataR{camIDIdx,seqNumIdx} = PointDataR{camIDIdx,seqNumIdx}(selPtsR,:);
    PointDataB{camIDIdx,seqNumIdx} = PointDataB{camIDIdx,seqNumIdx}(selPtsB,:);
    
    nextImage();
end

function rejectImage(obj,evnt)
    global PointDataR PointDataB camIDIdx seqNumIdx;
    
    PointDataR{camIDIdx,seqNumIdx} = zeros(0,2);
    PointDataB{camIDIdx,seqNumIdx} = zeros(0,2);
    
    nextImage();
end

