%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [p, v] = convergentAvg(P, V)
% Given a set of 2D vectors with directions V and start points P all
% converging toward a single point, find a vector with the average
% direction of all input vectors and position p such that it also points
% toward the convergence point.
%
%   P and V are n-by-2 matrices.

    numPts = size(P,1);

    P = [P, zeros(numPts,1)];
    V = [V, zeros(numPts,1)];

    normalize = 1;
    if (normalize)
        [M,Ps] = isoNorm2D([P(:,1:2), ones(numPts,1)]);
        Ps = [Ps(:,1:2), zeros(numPts,1)];
        Vs = V * M(1,1);

        P = Ps;
        V = Vs;
    end
    
    cr = sum(cross(P, V), 1) / numPts;

    v = sum(V,1) / numPts;

    % Need additional constraint because vcross is singular. Constrain the
    % origin point pbar to lie along the line passing through the centroid of
    % the points and perpendicular to vbar
    c = sum(P, 1) / numPts;
    l = [v(1); v(2); -v(2) * c(2) - v(1) * c(1)];
    %l = l / l(3)

    A = [-v(2), v(1); l(1), l(2)];
    b = [-cr(3); -l(3)];

    v = v(1:2)';
    p = A \ b;

    if (normalize)
        v = v / M(1,1);
        p = M \ [p;1];
        p = p / p(3);
        p = p(1:2);
    end
    