%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Preprocess images to detect and refine point positions

fileExt = 'pgm';
sep = '_';

allImages = dir(['./*' sep '*.' fileExt]);

% Find all the camera ids and sequence numbers
[camIDs, seqNums] = getCameraIDs(allImages, sep);

% cell(<camera>, <sequence_no>) is N-by-2 array of points
PointDataR = cell(numel(camIDs), numel(seqNums));
PointDataB = cell(numel(camIDs), numel(seqNums));


for i=1:numel(camIDs)
    for j=1:numel(seqNums)
        % Just scan every combination
        imgFile = [seqNums{j} sep camIDs{i} '.' fileExt];

        if exist(imgFile, 'file')
            disp(['Found file ' imgFile]);
        
            I = imread(imgFile);
            I(1,1:4) = 0; % Ignore the embedded timestamp (PointGrey cameras)

            [Xr, Xb] = detectGridPtsProc(I);

            PointDataR{i,j} = Xr - 1; % Convert to zero-based indexing
            PointDataB{i,j} = Xb - 1;
        else
            PointDataR{i,j} = zeros(0,2);
            PointDataB{i,j} = zeros(0,2);
        end
    end
end

save('GridPointData.mat', 'PointDataR', 'PointDataB', 'allImages', 'camIDs', 'seqNums', 'fileExt', 'sep');
