%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Xr, Xb] = detectGridPtsProc(I)
% Detect and refine grid points. Hardcoded parameter settings for now.

    [m,n] = size(I);

    % Bayer CFA pattern for PointGrey Flea2 (0.8MP)
    redRows = 1:2:m;
    redCols = 2:2:n;
    blueRows = 2:2:m;
    blueCols = 1:2:n;
    
    % Bayer CFA pattern for PointGrey Grasshopper (5MP)
%     redRows = 2:2:m;
%     redCols = 1:2:n;
%     blueRows = 1:2:m;
%     blueCols = 2:2:n;
    
    Ir = I(redRows, redCols);
    Ib = I(blueRows, blueCols);

    [Xr,Yr] = findPeaks4(Ir, 5, 25, 4, 15, 4);
    [Xb,Yb] = findPeaks4(Ib, 5, 25, 4, 15, 4);
    
    Xr = (Xr - 1) * 2 + redCols(1);
    Yr = (Yr - 1) * 2 + redRows(1);

    Xb = (Xb - 1) * 2 + blueCols(1);
    Yb = (Yb - 1) * 2 + blueRows(1);

    
%     figure; hold on;
%     imagesc(I); colormap gray;
%     plot(Xr, Yr, 'r*');
%     plot(Xb, Yb, 'bo');
%     xlim([0.5, size(I,2) + 0.5]);
%     ylim([0.5, size(I,1) + 0.5]);
%     set(gca,'YDir','reverse')
%     hold off;
%     Xr = [Xr,Yr];
%     Xb = [Xb,Yb];
%     return;

    [Xropt, Yropt] = findOptimPoints(I, Xr, Yr, redRows, redCols, 17);
    [Xbopt, Ybopt] = findOptimPoints(I, Xb, Yb, blueRows, blueCols, 17);

    %Sorry for reusing this variable name
    Xr = [Xropt', Yropt'];
    Xb = [Xbopt', Ybopt'];

        
%     figure; hold on;
%     imagesc(I); colormap gray;
%     plot(Xr(:,1), Xr(:,2), 'r*');
%     plot(Xb(:,1), Xb(:,2), 'bo');
%     xlim([0.5, size(I,2) + 0.5]);
%     ylim([0.5, size(I,1) + 0.5]);
%     set(gca,'YDir','reverse')
%     hold off;
    