%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x] = estConvergentPoint3D(P, V, K)
    Vnorms = sqrt(sum(V .* V, 2));
    
    % Exclude points for which direction vector V is zero
    useRows = (Vnorms >= 1e-9);
    V = V(useRows,:);
    P = P(useRows,:);

    n = size(P,1);
    
    PP = P + V;
    
    P3D = (K \ ([P, ones(n,1)])')';
    %P3D = P3D ./ (sqrt(sum(P3D .* P3D,2)) * ones(1,3));
    
    PP3D = (K \ ([PP, ones(n,1)])')';
    %PP3D = PP3D ./ (sqrt(sum(PP3D .* PP3D,2)) * ones(1,3));
    
    % Refraction plane normals
    RefPlanes = cross(P3D, PP3D, 2);    
    
    % Normalization step
    RefPlanes = RefPlanes ./ (sqrt(sum(RefPlanes .* RefPlanes, 2)) * ones(1,3));
    
    [U,S,Vs] = svd(RefPlanes);
    A = Vs(:,end);
    x = K * A;
    x = x / x(3);
    