%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Input image I is 2D grayscale
function [X,Y] = findOptimPoints(I, Xinit, Yinit, subRows, subCols, wSize)
    SetDefaultValue(6, 'wSize', 11);

    numPeaks = length(Xinit);

    [Iheight, Iwidth] = size(I);
    
    ipWidth = wSize;
    ipHeight = wSize;
    
    X = [];
    Y = [];
    
    for i=1:numPeaks
        disp(['Optimizing point ' num2str(i)]);

        x = Xinit(i) - floor(ipWidth / 2);
        y = Yinit(i) - floor(ipHeight / 2);
        
        if (x < 1) || (y < 1) || (x + ipWidth > Iwidth) || (y + ipHeight > Iheight)
            disp('Point too close to edge, skipping');
            continue;
        end
        
        Ip = double(I(y:y+ipHeight, x:x+ipWidth));

        [m,n] = size(Ip);
        rows = subRows - (y - 1);
        rows = rows((rows >= 1) & (rows <= m));
        cols = subCols - (x - 1);
        cols = cols((cols >= 1) & (cols <= n));

        [X(end+1), Y(end+1)] = fitPointEllipse(Ip, rows, cols);

        X(end) = X(end) + x - 1;
        Y(end) = Y(end) + y - 1;
    end
    