%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Look for local maxima in an image. A heuristic equation with various
% thresholds is used, see below.
%
% Maxima points must be at least a certain distance away from each other, 
% specified by the margin argument in pixels. If two points are too close, 
% one of them is removed.
function [X,Y] = findPeaks4(I, diffThreshold, absThreshold, margin, absThreshold2, toler)
SetDefaultValue(4, 'margin', 1);
SetDefaultValue(5, 'absThreshold2', -1);
SetDefaultValue(6, 'toler', 0);

[m,n] = size(I);

nhood = ones(3);
nhood(2,2) = 0;

nhood2 = ones(5);
nhood2(2:4, 2:4) = 0;

nhood3 = ones(7);
nhood3(2:6, 2:6) = 0;

Id = imdilate(I, nhood);
Id2 = imdilate(I, nhood2);
Id3 = imdilate(I, nhood3);

peaks = (I > absThreshold) & ((I > (Id + diffThreshold)) | ((I >= Id - toler) & (I > (Id2 + diffThreshold))) | ((I >= Id - toler) & (I >= Id2 - toler) & (I > Id3 + diffThreshold))) & (Id > absThreshold2);

[Y,X] = find(peaks);

% Remove some peaks if they are right next to each other
for i=1:length(X)
    peaks(max(1,Y(i)-margin):min(m,Y(i)+margin), max(1,X(i)-margin):min(n,X(i)+margin)) = 0;
    peaks(Y(i), X(i)) = 1;
end
[Y,X] = find(peaks);