%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Fit a 2D elliptical gaussian onto the image of a point
%   I is a matrix of doubles
%   rows and cols are index vectors to indicate which pixels of I to
%   consider
function [x,y,cx,cy,err] = fitPointEllipse(I, rows, cols)
    [m,n] = size(I);
    [X,Y] = meshgrid(cols, rows);

    Isub = I(rows, cols);
    
    imgsum = sum(Isub(:));
    imgmax = max(Isub(:));

    Isc = Isub / imgsum;
    
    cx = sum(sum(X .* Isc));
    cy = sum(sum(Y .* Isc));

    % Estimate covariance matrix
    Xs = X - cx;
    Ys = Y - cy;
    Cov1 = (Xs.^2).*Isc;
    Cov2 = Xs.*Ys.*Isc;
    Cov3 = (Ys.^2).*Isc;
    sumCov1 = sum(Cov1(:));
    sumCov2 = sum(Cov2(:));
    sumCov3 = sum(Cov3(:));
    C = [sumCov1, sumCov2;
        sumCov2, sumCov3];
    
    [V,D] = eig(C);
    angle = atan2(V(2,1), V(1,1));
    height = imgmax * sqrt(2 * pi * D(1,1));
    
    x0 = [cx, cy, sqrt(D(1,1)), sqrt(D(2,2)), angle, height, 0, 0];
    
    optimFunc = @(x) gaussianDiff2DEllipse(x(1), x(2), x(3), x(4), x(5), x(6), (I - x(7)), rows, cols);

    %options = optimset('Display', 'iter', 'MaxFunEvals', 40000, 'MaxIter', 40000);
    options = optimset('MaxFunEvals', 40000, 'MaxIter', 40000);
    [res, fval] = fminsearch(optimFunc, x0, options);
    
    x = res(1);
    y = res(2);
    err = fval;
    
