%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [v, normImg2D, imgdiff] = gaussianDiff2DEllipse(x, y, sx, sy, theta, i, I, rows, cols)
% Coordinates x and y start from (1,1) being the upper-left corner,
% increasing to the right and down.
% theta gives angle of rotation to bring the gaussian into canonical
% coordinates. Counterclockwise.
% sx, sy are standard deviation in the canonical x and y axes
% i is height of the gaussian

    numRows = length(rows);
    numCols = length(cols);
    
    [X,Y] = meshgrid(cols, rows);
    
    X = X - x;
    Y = Y - y;
    
    R = [cos(theta), sin(theta);
        -sin(theta), cos(theta)];
    
    tmp = R * [X(:), Y(:)]';
    tmp(2,:) = tmp(2,:) * sx / sy;
    
    X = reshape(tmp(1,:), numRows, numCols);
    Y = reshape(tmp(2,:), numRows, numCols);
    
    dist = sqrt(X.^2 + Y.^2);
    
    normImg2D = i*normpdf(dist, 0, sx);
    imgdiff = double(I(rows, cols)) - normImg2D;
    
    v = sum(sum(imgdiff.^2));
    %v = imgdiff(:);
    %v = sum(sum(abs(imgdiff)));
  