%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = lcaCalib()

    ptsDataFile = 'GridPointData.mat';
    alignedPtsDataFile = 'AlignedGridPointData.mat';
    calibDir = 'result';
    
    initialWorkingDirectory = pwd;

    % Run detectGridPts2.m to generate this data
    load(ptsDataFile, 'camIDs', 'seqNums', 'fileExt', 'sep');
    
    if length(camIDs) > 1
        error(['File ' pointsData ' contains data for more than one camera. Intrinsic calib is for one camera at a time.']);
    end
    
    format_image = fileExt;
    calib_name = ['cam' camIDs{1} '_n'];
    
    mkdir(calibDir);
    
    load(alignedPtsDataFile, 'AssignedGridR', 'AssignedGridB', 'AssignedPointsR', 'AssignedPointsB', 'AssignedPointFiles');
    
    imgNum = 1;
    for i=1:numel(camIDs)
        for j=1:numel(seqNums)
            imgFile = [seqNums{j} sep camIDs{i} '.' fileExt]
    
            if ~exist(imgFile, 'file')
                disp('File doesn''t exist');
                break;
            end
            
            found = false;
            for k=1:numel(AssignedPointFiles)
                if strcmp(imgFile, AssignedPointFiles{k})
                    found = true;
                    break;
                end
            end
            
            if ~found
                disp('Not found in AssignedPointFiles');
                break;
            end
            
            disp(['Found file ' imgFile]);

            copyfile(imgFile, [calibDir '/' calib_name num2str(imgNum) '.' format_image]);
            imgNum = imgNum + 1;
        end
    end
    
    doTheCalib(AssignedGridR, AssignedPointsR, calib_name, format_image, initialWorkingDirectory, calibDir);
    movefile([calibDir '/Calib_Results.mat'], [calibDir '/Calib_Results_red.mat']);

    doTheCalib(AssignedGridB, AssignedPointsB, calib_name, format_image, initialWorkingDirectory, calibDir);
    movefile([calibDir '/Calib_Results.mat'], [calibDir '/Calib_Results_blue.mat']);



function [] = doTheCalib(assignedGrids, detectedPoints, calib_name, format_image, initialWorkingDirectory, folder)
    num_imgs = numel(assignedGrids);

    for i=1:num_imgs
        eval(['x_' num2str(i) ' = detectedPoints{ ' num2str(i) '}'';']);
        
        S = assignedGrids{i};
        Np = size(S,1);

        eval(['X_' num2str(i) ' = [S'';zeros(1,Np)];']);
    end

    % Loads the images:

    cd(folder);
    ima_read_calib_no_read;

    % Set the toolbox not to prompt the user (choose default values)

    dont_ask = 1;

    % Run the main calibration routine:

    go_calib_optim_no_read;

    % Shows the extrinsic parameters:

    ext_calib;

    % Reprojection on the original images:

%     reproject_calib_no_read;

    % Saves the results into a file called Calib_Results.mat:

    saving_calib;

    %close all;

    % Set the toolbox to normal mode of operation again:

    dont_ask =  0;

    cd(initialWorkingDirectory);


