%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [y] = refrac2DImg2(sx, sy, e, r1, r2, t)
% Refractive projection through two interfaces parallel to the y axis, the 
% first one at x=0 and the second one at x=-t.
% (sx,sy) are the coords of the point S undergoing refraction
% e is x-coordinate of the eye-point, assumed positive
% r1 is the refractive index for the first refraction (at x=0) and r2 is
% for the second refraction (at x=-t)
%
% See backProj2 for more details

if t == 0
    y = refrac2DImg(sx, sy, e, r2);
    return;
end

% Initialize with perspective projection approximation, then find the 
% projection point via optimization
e = abs(e);
sx = abs(sx);

if sx == 0
    y = sy;
    return;
end

if 1
    % A closer approximation that assumes r1 > r2 !!
    ylb = sy * (r2 * e) / (r2 * e + sx);

    denom = (r2 * e + sx)^2 + (1 - r1^2) * sy^2;
    if denom <= 0
        yub = sy;
    else
        yub = r1 * sy * e / sqrt(denom);
    end

    if sy < 0
        temp = ylb;
        ylb = yub;
        yub = temp;
    end
else
    y0 = sy * e / (e + sx);
    y0Proj = backProj2(y0, sx, e, t, r1, r2);

    if y0Proj < sy
        ylb = y0;
        yub = y0 + (sy - y0Proj);
    else
        yub = y0;
        ylb = y0 - (y0Proj - sy);
    end
end

optFunc = @(x) sy - backProj2(x, sx, e, t, r1, r2);
%options = optimset('Display', 'iter');

if abs(sx) < t
    disp('3D point is inside the glass.');
    y = 0;
elseif yub == ylb
    y = ylb;
elseif (sign(optFunc(ylb)) == sign(optFunc(yub)))
    disp('Possible error in inputs');
    sx
    sy
    e
    r1
    r2
    t
    yub
    ylb
    
    y = 0; % Error in inputs, probably. Hack to avoid crashing.
else
    y = fzero(optFunc, [ylb, yub]);
end

