################################################################################
# Thesis title: Underwater Camera Calibration and 3D Reconstruction
# http://hdl.handle.net/10402/era.38695
# Author: Timothy Yau
################################################################################

use warnings;

open FH, "../calib/extrinsic/list.txt" or die "Couldn't find list file in calibration directory";

@camIDs = ();
while (<FH>) {
  $_ =~ /^.+_(\d+)\./;
  push @camIDs, $1;
}

foreach $id (@camIDs) {
  printf("Found Camera %s\n", $id);
}

close FH;

open FH, "./list.txt" or die "Couldn't find list file in current directory";
@imageNames = <FH>;
close FH;

open FH, ">./newlist.txt" or die "Couldn't open new list file for writing";

%idxMap = ();

for ($i = 0; $i <= $#camIDs; $i++) {
  $id = $camIDs[$i];

  for ($j = 0; $j <= $#imageNames; $j++) {
    $imgName = $imageNames[$j];

    if ($imgName =~ /_${id}\./) {
      printf(FH $imgName);

      $idxMap{$j} = $i;

      last;
    }
  }
}

close FH;

print "Image index reordering table:\n";
use Data::Dumper;
print Dumper(\%idxMap);

# matchGrid[i][j] is an array reference to the match list for cameras i and j, indices after reordering, and i < j
@matchGrid = ();

for ($i = 0; $i <= $#camIDs; $i++) {
  @tmp = ();
  for ($j = 0; $j <= $#camIDs; $j++) {
    push @tmp, [];
  }

  push @matchGrid, [ @tmp ];
}

#print Dumper($matchGrid[0][0]);

open FH, "./matches.init.txt" or die "Couldn't open matches file";
while (<FH>) {
  $_ =~ /(\d+) (\d+)/;
  $origCamIdx1 = $1;
  $origCamIdx2 = $2;

  $numMatches = <FH>;

  $newCamIdx1 = $idxMap{$origCamIdx1};
  $newCamIdx2 = $idxMap{$origCamIdx2};

  my @matchList = ();
  for ($i=0; $i < $numMatches; $i++) {
    $_ = <FH>;
    $_ =~ /(\d+) (\d+)/;
    
    $keyIdx1 = $1;
    $keyIdx2 = $2;

    # Must keep the camera indexes in ascending order, so we need to switch the two lists
    if ($newCamIdx1 > $newCamIdx2) {
       $tmp = $keyIdx1;
       $keyIdx1 = $keyIdx2;
       $keyIdx2 = $tmp;
    }

    push @matchList, [$keyIdx1, $keyIdx2];
  }

  @matchList = sort { $a->[0] <=> $b->[0] } @matchList;

  if ($newCamIdx1 > $newCamIdx2) {
    $matchGrid[$newCamIdx2][$newCamIdx1] = \@matchList;
  }
  else {
    $matchGrid[$newCamIdx1][$newCamIdx2] = \@matchList;
  }
}

close FH;

#print Dumper(@{$matchGrid[0][1]});

open FH2, ">./newmatches.init.txt" or die "Couldn't open new matches file for writing";

for ($j = 0; $j <= $#camIDs; $j++) {
  for ($i = 0; $i <= $#camIDs; $i++) {
    @matchList = @{$matchGrid[$i][$j]};

    if ($#matchList >= 0) {
      printf FH2 "%d %d\n", ($i, $j);
      printf FH2 "%d\n", ($#matchList+1);

      for ($k = 0; $k <= $#matchList; $k++) {
        printf FH2 "%d %d\n", ($matchList[$k][0], $matchList[$k][1]);
      }
    }
  }
}

close FH2;

