%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [M, Xs] = isoNorm2D(X)
% Isotropic scaling to normalize data, as per Hartley & Zisserman,
% "Multiple View Geometry in Computer Vision (2nd ed)" section 4.4.4
%     M  is the matrix that maps X to the normalized points Xs

    n = size(X,1);

    % First we make all points the same scale (last coordinate = 1)
    X = X ./ repmat(X(:,3), 1,3);
    Xn = X(:,[1,2]);
    
    Xcentroid = mean(Xn,1);
    Xc = Xn - repmat(Xcentroid, n, 1);
    
    Xscale = sqrt(2) / sqrt(mean(sum(Xc .* Xc, 2)));
    Xs = [Xc * Xscale, ones(n,1)];

    M = [Xscale, 0, -Xcentroid(1)*Xscale;
        0, Xscale, -Xcentroid(2)*Xscale;
        0, 0, 1];
    