%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = printBundleCamsPoints(bundleFile, numCams, F, K, R, T, N, Mu, D, X, Color, KeyIdx, KeyCoord)

    fid = fopen(bundleFile, 'w');
    
    fprintf(fid, '# Bundle-refrac file v0.3\n');
    
    numPoints = size(X,2);
    fprintf(fid, '%i %i\n', numCams, numPoints);
        
    for i=1:numCams
        fprintf(fid, '%0.10e %0.10e %0.10e\n', F(i,1), K(i,1), K(i,2));
        
        fprintf(fid, '%0.10e %0.10e %0.10e\n', R(1,1,i), R(1,2,i), R(1,3,i));
        fprintf(fid, '%0.10e %0.10e %0.10e\n', R(2,1,i), R(2,2,i), R(2,3,i));
        fprintf(fid, '%0.10e %0.10e %0.10e\n', R(3,1,i), R(3,2,i), R(3,3,i));
        
        fprintf(fid, '%0.10e %0.10e %0.10e\n', T(i,1), T(i,2), T(i,3));
        
        fprintf(fid, '%0.10e %0.10e %0.10e\n', N(i,1), N(i,2), N(i,3));
        
        numRefrac = length(D{i});
        fprintf(fid, '%i', numRefrac);
        
        if numRefrac > 0
            for j=1:numRefrac
                fprintf(fid, ' %0.10e %0.10e', Mu{i}(j), D{i}(j));
            end

            fprintf(fid, ' %0.10e\n', Mu{i}(numRefrac+1));
        else
            fprintf(fid, '\n');
        end
    end
    
    for i=1:numPoints
        fprintf(fid, '%0.10e %0.10e %0.10e\n', X(1,i), X(2,i), X(3,i));
        fprintf(fid, '%d %d %d\n', Color(1,i), Color(2,i), Color(3,i));
        
        cams = find(KeyIdx(:,i) >= 0) - 1;
        fprintf(fid, '%i', length(cams));
        
        for j=1:length(cams)
            fprintf(fid, ' %i', cams(j));
            fprintf(fid, ' %i', KeyIdx(cams(j)+1,i));
            fprintf(fid, ' %0.4f %0.4f', KeyCoord(cams(j)+1,i,1), KeyCoord(cams(j)+1,i,2));
        end
        
        fprintf(fid, '\n');
    end
    
    fclose(fid);