%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [numCams, F, K, R, T, N, Mu, D] = readBundleCams(bundleFile)

    fid = fopen(bundleFile);
    
    fileHeading = fgets(fid);
    isBundleRefrac = ~isempty(strfind(fileHeading, 'Bundle-refrac'));
    
    numCams = fscanf(fid, '%i', 1);
    fscanf(fid, '%i', 1);
    
    F = zeros(numCams, 1);
    K = zeros(numCams, 2);
    R = zeros(3, 3, numCams);
    T = zeros(numCams, 3);
    
    N = zeros(numCams, 3);
    Mu = cell(numCams, 1);
    D = cell(numCams, 1);
    
    for i=1:numCams
        F(i) = fscanf(fid, '%f', 1);
        K(i,:) = fscanf(fid, '%f', [1,2]);
        R(:,:,i) = fscanf(fid, '%f', [3,3]);
        R(:,:,i) = R(:,:,i)';
        T(i,:) = fscanf(fid, '%f', [1,3]);
        
        if isBundleRefrac
            N(i,:) = fscanf(fid, '%f', [1,3]);
            numRefrac = fscanf(fid, '%i', 1);
            
            if numRefrac > 0
                Mu{i} = zeros(numRefrac+1, 1);
                D{i} = zeros(numRefrac, 1);

                for j=1:numRefrac
                    Mu{i}(j) = fscanf(fid, '%f', 1);
                    D{i}(j) = fscanf(fid, '%f', 1);
                end

                Mu{i}(numRefrac+1) = fscanf(fid, '%f', 1);
            else
                Mu{i} = zeros(0);
                D{i} = zeros(0);
            end
        end
    end
    
    fclose(fid);