%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [X, Color, KeyIdx, KeyCoord] = readBundlePoints(bundleFile)
% KeyIdx(camIdx, ptIdx) gives the key index of the given point for the
% given camera, or -1 if there was no keypoint corresponding to this point.
% KeyCoord(camIdx, ptIdx, :) is a 2-vector giving the image coordinates of
% that keypoint, or zeros of no keypoint (see above).

    disp(['bundle file: ' bundleFile]);

    fid = fopen(bundleFile);
    
    fileHeading = fgets(fid);
    isBundleRefrac = ~isempty(strfind(fileHeading, 'Bundle-refrac'));
    
    numCams = fscanf(fid, '%i', 1);
    numPts = fscanf(fid, '%i', 1);
    fgets(fid);
    
    for i=1:numCams
        fgetl(fid);
        fgetl(fid); fgetl(fid); fgetl(fid);
        fgetl(fid);
        
        if isBundleRefrac
            fgetl(fid); fgetl(fid);
        end
    end
    
    X = zeros(3, numPts);
    Color = zeros(3, numPts);
    KeyIdx = ones(numCams, numPts) * -1;
    KeyCoord = zeros(numCams, numPts, 2);
    
    for i=1:numPts
        X(:,i) = fscanf(fid, '%f', 3);
        Color(:,i) = fscanf(fid, '%i', 3);
        
        numViews = fscanf(fid, '%i', 1);
        
        for j=1:numViews
            camIdx = fscanf(fid, '%i', 1);
            KeyIdx(camIdx+1, i) = fscanf(fid, '%i', 1);
            KeyCoord(camIdx+1, i, :) = fscanf(fid, '%f', 2);
        end
    end
    
    fclose(fid);
    