%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Xout, KK] = undistortAndCenterPoints(X, fc, cc, kc, alpha_c, imgWidth, imgHeight)
    % Normalize and undistort image points, then project them again using a
    % simplified intrinsic parameter matrix (see below).

    if ~exist('fc')|~exist('cc')|~exist('kc')|~exist('alpha_c'),
        fprintf(1,'No intrinsic camera parameters available.\n');
        return;
    end;
    
    fcMean = mean(fc);
    KK = [fcMean 0 ((imgWidth-1)/2);0 fcMean ((imgHeight-1)/2); 0 0 1];

    Xout = normalize(X',fc,cc,kc,alpha_c)';
    Xout = (KK * [Xout'; ones(1,size(X,1))])';
    Xout = Xout(:,1:2);
    