%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = undistortImages(bundleFile)

    margin = 75; % Black margin width, in pixels

    [numCams, F, K, R, T, N, Mu, D] = readBundleCams(bundleFile);
    
    fid = fopen('list.txt', 'r');
    camIdx = 1;
    
    edgeSaveDir = 'edge_undist';
    if ~exist(edgeSaveDir, 'dir')
        mkdir(edgeSaveDir);
    end
    
    figure;
    
    while 1
        tline = fgetl(fid);
        if ~ischar(tline), break, end

        [calibFile, imgName, camID, imgFile] = getIntrinsicCalib(tline);
        maskFile = ['edges/' imgName '.pgm'];
        
        disp(['Camera: ' camID]);
        disp(['Loading calib file: ' calibFile]);

        load(calibFile, 'fc', 'cc', 'kc', 'alpha_c', 'nx', 'ny');
        nxNew = nx + 2*margin;
        nyNew = ny + 2*margin;
        ccNew = [((nxNew-1)/2), ((nyNew-1)/2)];
        
        fcB = F(camIdx,1);
        kB = K(camIdx,:);

        KK = [fc(1) 0 ccNew(1);0 fc(2) ccNew(2) ; 0 0 1];
        
        I = double(imread(imgFile));
        
        if exist(maskFile, 'file')
            disp(['Found mask file ' maskFile]);
            Imask = double(imread(maskFile));
        else
            Imask = [];
        end
        
        axis equal; image(I/255); title('Original'); drawnow;
        
        Irec = rectWithBundler(I(:,:,1), eye(3), fc, cc, kc, alpha_c, KK, nyNew, nxNew, fcB, kB);
        Irec = cat(3, Irec, rectWithBundler(I(:,:,2), eye(3), fc, cc, kc, alpha_c, KK, nyNew, nxNew, fcB, kB));
        Irec = cat(3, Irec, rectWithBundler(I(:,:,3), eye(3), fc, cc, kc, alpha_c, KK, nyNew, nxNew, fcB, kB));
        
        Irec(Irec > 255) = 255; % Sometimes values might be higher due to numerical issues?
        
        assignin('base', 'Irec', Irec);
        
        axis equal; image(Irec/255); title('Undistorted'); drawnow;
        
        imwrite(Irec/255, [imgName '.jpg.undist'], 'jpg');
        
        if ~isempty(Imask)
            Imaskrec = rectWithBundler(Imask, eye(3), fc, cc, kc, alpha_c, KK, nyNew, nxNew, fcB, kB);
            Imaskrec(Imaskrec > 127) = 255;
            Imaskrec(Imaskrec <= 127) = 0;

            edgeFile = sprintf([edgeSaveDir '/%08d.pgm'], camIdx-1);
            imwrite(Imaskrec/255, edgeFile, 'pgm');
        end
        
        camIdx = camIdx + 1;
    end
    
    close;
    
    [X, Color, KeyIdx, KeyCoord] = readBundlePoints(bundleFile);
    
    % Produce a temporary bundle file with distortion zeroed, so that we
    % can use Bundle2PMVS and it won't do radial distortion again
    K(:) = 0;
    printBundleCamsPoints('bundle_tmp.out', numCams, F, K, R, T, N, Mu, D, X, Color, KeyIdx, KeyCoord);
    