%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thesis title: Underwater Camera Calibration and 3D Reconstruction
% http://hdl.handle.net/10402/era.38695
% Author: Timothy Yau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen('list.txt', 'r');

while 1
  tline = fgetl(fid);
  if ~ischar(tline), break, end
  
  [calibFile, imgName, camID] = getIntrinsicCalib(tline);
    
  disp(['Camera: ' camID]);
  disp(['Loading calib file: ' calibFile]);
  
  load(calibFile, 'fc', 'cc', 'kc', 'alpha_c', 'nx', 'ny');
  imgWidth = nx;
  imgHeight = ny;
  
  system(['gzip -d ' imgName '.key.gz']);
  
  fid2 = fopen([imgName '.key']);
  A = fscanf(fid2, '%d', 2);
  
  numKeys = A(1);
  keyLength = A(2);
  disp(['Found ' num2str(numKeys) ' keys, length ' num2str(keyLength)]);
  
  fid3 = fopen([imgName '_new.key'], 'w');
  fprintf(fid3, '%d %d\n', numKeys, keyLength);
  
  for i=1:numKeys
      A = fscanf(fid2, '%f', 4);
      desc = fscanf(fid2, '%d', keyLength);
      
      y = A(1);
      x = A(2);
      
      [Xout] = undistortAndCenterPoints([x y], fc, cc, kc, alpha_c, imgWidth, imgHeight);
      
      xNew = Xout(1);
      yNew = Xout(2);
      
      fprintf(fid3, '%f %f %f %f\n', yNew, xNew, A(3), A(4));
      
      for j=1:int32(ceil(keyLength/20))
          fprintf(fid3, ' %d', desc(((j-1)*20+1) : min([j*20 keyLength])));
          fprintf(fid3, '\n');
      end
  end
  
  fclose(fid3);
  fclose(fid2);
  
  system(['rm -f ' imgName '.key']);
  system(['mv ' imgName '_new.key ' imgName '.key']);
  system(['gzip ' imgName '.key']);
end

fclose(fid);


